# -----------------------------------------------------------------------------
# Clouds that slowly drift across the screen in the same direction as the
# waves. The higher up they are the slower they drift.
#
# Copyright 2005 Ron Dippold
# -----------------------------------------------------------------------------

import pygame.sprite, pygame.image
from   pygame.locals import *
import random

from ImageSet import ImageSet
import Util


def imagesmack( _frames ):
	"""
	Convert the series of frames from greyscale levels to
	being all-white, but with the old grey value as the per-pixel
	alpha.
	"""

	# convert to having per-pixel alpha
	frames = [ frame.convert_alpha() for frame in _frames ]

	# For each frame, get the grey value at every pixel and
	# make that the alpha value for a white pixel instead
	for frame in frames:
		for y in range(frame.get_height()):
			for x in range( frame.get_width() ):
				val = frame.get_at( (x,y) )[0]
				frame.set_at( (x,y), (255,255,255,val/2) )
		frame.set_alpha( 255, RLEACCEL )


	# Create flipped versions of all the frames for more variety
	more = []
	for frame in frames:
		more += [
			frame,
			pygame.transform.flip( frame, True, False ),
			pygame.transform.flip( frame, False, True ),
			pygame.transform.flip( frame, True, True ) ]
		
	return more
	

# -----------------------------------------------------------------------------
#
# class Cloud
#
# -----------------------------------------------------------------------------

class Cloud( pygame.sprite.Sprite ):
	"""
	Fluffly clouds that slowly drift across the screen in the same direction
	as the waves and eventually reappear if they go off screen.

	"""

	# Our set of clouds
	imageset = None

	# if we go outside here, we're dead
	clip = None

	# A helper to get the speed and wave top position
	WaveHelper = None
	

	#@staticmethod
	def set_clip( _clip ):
		Cloud.clip = _clip
	# Pre python 2.4 syntax to make it static - sigh
	set_clip = staticmethod( set_clip )

	#@staticmethod
	def set_WaveHelper( _helper ):
		Cloud.WaveHelper = _helper
	# Pre python 2.4 syntax to make it static - sigh
	set_WaveHelper = staticmethod( set_WaveHelper )

	#@staticmethod
	def preload():
		Cloud.imageset = Util.imageset_loader( 'cloud', _callback=imagesmack )

	# Pre python 2.4 syntax to make it static - sigh
	preload = staticmethod( preload )


	# --------------------------
	# New Cloud
	# --------------------------
	def __init__( self ):
		"""
		Make a new random cloud

		"""
		# Always init the sprite
		pygame.sprite.Sprite.__init__( self )

		if not Cloud.imageset:
			Cloud.preload()

		self.choose_look_and_location( True )

	def choose_look_and_location( self, _randomx=False ):
		"""
		Choose a new random frame and position for the cloud.
		if _randx is True, we start the cloud anywhere (for start of game).
		Otherwise, it has to come in on left or right
		"""
		# y position is random
		self.pos = [0,0]
		self.pos[1] = random.randrange( 0, self.WaveHelper.min_top()*2/3 )
		
		# Pick the imageset based on our size, then choose a random frame
		self.imageset = Cloud.imageset.shared_copy()
		self.image, self.rect = self.imageset.random_frame( (0,0) )


		if _randomx:
			self.pos[0] = random.randrange( self.clip.left, self.clip.right )
		else:
			# start on the left if waves are going right, or vice versa
			if self.WaveHelper.get_xspeed()<0:
				self.pos[0] = self.clip.right + self.rect.width/2 + random.randrange(10)
			else:
				self.pos[0] = self.clip.left - self.rect.width/2 - random.randrange(10)

		self.rect.center = self.pos

	#
	# Move it depending on how much time has passed
	#
	def update( self, seconds ):
		# Speed is wave speed divided by how high up we are
		speedx = Cloud.WaveHelper.get_xspeed()
		speedx *= self.pos[1]/float(Cloud.WaveHelper.min_top())

		# actually move it
		self.pos[0] += speedx * seconds
		self.image, self.rect = self.imageset.current_frame( 0, self.pos )

		self.rect.center = self.pos

		# If we're no longer in the play area, loop back around
		if not self.rect.colliderect( Cloud.clip ):
			self.choose_look_and_location()
				
