# ------------------------------------------------------------------
# The various types of text sprite we might want to put up on the
# screen - from unchanging TextSprite to uberfancy WavyText
#
# Copyright 2005 Ron Dippold
# ------------------------------------------------------------------

import pygame.font, pygame.sprite
from pygame.locals import *

from FixedFont import FixedFont

# ------------------------------------------------------------------
# TextSprite classes
# ------------------------------------------------------------------

class TextSprite( pygame.sprite.Sprite ):
	"""
	Create a simple unchanging sprite from some text.
	"""

	def __init__( self, _text, _points, _color, _center, _bgcolor=None ):
		"""
		_text is the text to render
		_point is the font size (ex 36)
		_color is the (r,g,b) color for the text
		_center is where to place the text
		_bgcolor is (r,g,b) or None for transparent
		"""
		pygame.sprite.Sprite.__init__( self )

		# save the basics
		self.points = _points
		self.text = _text
		self.color = _color
		self.bgcolor = _bgcolor
		self.center = _center

		# Render it normally once at least
		self.render()

	def set_text( self, _text ):
		self.text = _text

	def set_color( self, _fgcolor, _bgcolor=None ):
		self.color = _fgcolor
		self.bgcolor = _bgcolor

	def render( self ):
		font = pygame.font.SysFont( None, self.points )
		if self.bgcolor:
			self.image = font.render( self.text, True, self.color, self.bgcolor)
		else:
			self.image = font.render( self.text, True, self.color )
		self.rect = self.image.get_rect()
		self.rect.center = self.center

	def get_size( self ):
		return 

	# has no update() function


class FixedFontText( TextSprite ):
	"""
	A sprite of fixed size text (used for score)
	"""

	def __init__( self, _fontname, _points, _text, _color, _center ):

		self.font = FixedFont( _fontname, _points, True, _color )
		TextSprite.__init__( self, _text, _points, _color, _center )

	def render( self ):
		self.image = self.font.render( self.text )
		self.rect = self.image.get_rect()
		self.rect.center = self.center

	def set_text( self, _text ):
		self.text = _text
		self.render()
	

class BlinkText( TextSprite ):
	"""
	Create a text sprite that blinks between colors.
	"""

	def __init__( self, _text, _points, _colors, _center, _speed=1.0 ):
		"""
		_text is the text to render
		_point is the font size (ex 36)
		_colors is tuple of (r,g,b) colors to switch between
		_center is where to place the text
		_speed is how many seconds between color switches
		"""

		TextSprite.__init__( self, _text, _points, _colors[0], _center )

		self.speed = _speed
		self.colors = _colors
		self.colors_idx = 0
		self.time = 0.0

	def update( self, seconds ):
		"""Every second switch to the next color"""
		self.time += seconds
		if self.time >= self.speed:
			self.time -= self.speed
			# move to next color
			self.colors_idx = (  self.colors_idx + 1 ) % len( self.colors )
			self.color = self.colors[self.colors_idx] 
			self.render( )
		

class RainbowText( TextSprite ):
	"""
	Text sprite made of multicolored strips that may rotate.
	"""

	def __init__( self, _text, _points, _colors, _center, _speed=None ):
		"""
		_text is the text to render
		_point is the font size (ex 36)
		_colors is tuple of (r,g,b) colors to use for striping
		_center is where to place the text
		_speed is how many pixels per second to rotate
		"""		

		# We could save just the band, but this lets us rotate it
		font = pygame.font.SysFont( None, _points )
		self.images = [ font.render( _text, True, color ) for color in _colors ]

		# This should not be needed, but we get very strange things happening when
		# we try to convert one of the font surfaces
		self.image = font.render( _text, True, (255,0,0) )
		self.image.set_colorkey( self.image.get_at((0,0)), RLEACCEL )

		# Now init the base class
		TextSprite.__init__( self, _text, _points, 0, _center )

		self.speed = _speed
		self.pos = 0.0

	def render( self, _offset=0 ):
		dy = self.image.get_height()/len(self.images)
		idx = (_offset/dy) - 1
		y = (_offset%dy) - dy
		while y<self.image.get_height():
			if idx<0:
				idx = len(self.images)-1
			self.image.blit( self.images[idx], (0,y), pygame.Rect(0,y,self.image.get_width(),y+dy) )
			y += dy
			idx -=1
		self.rect = self.image.get_rect()
		self.rect.center = self.center

	def update( self, seconds ):
		if not self.speed:
			return
		oldpos = self.pos
		self.pos += self.speed * seconds
		if int(self.pos) != int(oldpos):
			if self.pos > self.image.get_height():
				self.pos -= self.image.get_height()
			self.render( int(self.pos) )

			
class WavyText( TextSprite ):
	"""
	Text sprite of two colors with 'waves' between the colors
	"""

	import math


	def __init__( self, _text, _points, _colortop, _colorbot, _center, _speed=None,
				  _waveheight=1, _colormid=None, _midheight=1 ):
		"""
		_text is the text to render
		_point is the font size (ex 36)
		_color1 and _color2 are the two colors to use
		_center is where to place the text
		_speed is how many pixels per second to move
		_zone is how many pixels up and down the wave extends
		"""		

		# save passed info
		self.colortop = _colortop
		self.colorbot = _colorbot
		self.waveheight = _waveheight
		self.speed = _speed
		self.pos = 0.0
		self.colormid = _colormid
		if self.colormid:
			self.midheight = _midheight
		else:
			self.midheight = 0

		# Make a font mask that we can stamp black over the image - black is the
		# real image (around the font), just let the white be transparent
		font = pygame.font.SysFont( None, _points )
		self.mask = font.render( _text, False, (255,255,255), (0,0,0) ).convert()
		self.mask.set_colorkey( (255,255,255) )
		del font

		# Just make a copy of the mask right now - we're going to fill it with lines
		# Make it transparent, but do NOT do RLEACCEL
		self.image = self.mask.convert()
		self.image.set_colorkey( self.image.get_at((0,0)) )

		# Get our dimensions for easy use
		self.rect = self.image.get_rect()
		self.width = self.image.get_width()
		self.height = self.image.get_height()
		self.middle = self.height/2

		# The 'band' reaches from the wave height plus half of the
		#    midcolor height, since the midcolor is centered around the
		#    wave boundary.
		# the top of where we need to draw is the middle - half the band.
		# We're doing this to blit the minimum number of pixels necessary.
		self.midhalf = (self.midheight+1)/2
		self.bandhalf = self.waveheight + self.midhalf
		self.band = self.bandhalf*2
		self.bandtop = self.middle-self.bandhalf

		# Draw the top half in _colortop, the bottom half in _colorbot
		self.image.fill( _colortop, (0,0,self.width,self.middle) )
		self.image.fill( _colorbot, (0,self.middle,self.width,self.middle) )

		# Make a slice that's just one vertical line with a little _colortop
		# on top, a little _colorbot on bottom, and _colormid in between
		self.slice = pygame.Surface( (1,self.height) )
		self.slice.fill( _colortop, (0,0,1,self.middle) )
		self.slice.fill( _colorbot, (0,self.middle,1,self.middle) )
		if _colormid:
			self.slice.fill( _colormid, (0,self.middle-self.midhalf,1,self.midheight ) )

		# Precalculate the cosine table - just one cycle of it
		pi = WavyText.math.pi
		self.cos_table_width = 40
		self.cos_table = [ int(round( self.waveheight * \
									   WavyText.math.cos(2*pi*n/self.cos_table_width) ) ) \
							for n in range(self.cos_table_width) ]

		# Now init the base class
		TextSprite.__init__( self, _text, _points, 0, _center )

	def render( self, _offset=0 ):
		
		for x in range( self.width ):
			# Get the height for this x position from the cos_table
			y = self.cos_table[ (x+_offset)%self.cos_table_width ]

			# Blit both sides of the band from the slice into the destination,
			#   offset by the y - that offset makes the wavyness
			self.image.blit( self.slice, (x, self.bandtop), \
							 (0,self.bandtop+y, 1,self.band) )

		# Stamp the mask over it so we're just left with the font
		self.image.blit( self.mask, (0,0) )
		self.rect.center = self.center
		

	def update( self, seconds ):
		if not self.speed:
			return
		oldpos = self.pos
		self.pos += self.speed * seconds
		if int(self.pos) != int(oldpos):
			self.render( int(self.pos) )

			
	
