# =========================================================================
#
# TimedEvent - set up a set of timed callbacks.
#
# You add() callbacks, then pump() to cause them to happen.
#
# No worry about locking or threading, nothing happens till pump()
#
# Copyright 2005 Ron Dippold
# =========================================================================


# List of all known events
events = {}

# for anonymous events
unique_id = 0


def add( _seconds, _callback, _count=1, _data=None, _name=None ):
	"""
	Add a new event.
	_seconds = after how many seconds
	_callback = routine to call
	_count = invoke this many times, or 'forever'
	_data = any extra data to pass along to callback
	_name = unique ID for this, so you can remove() it

	The callback routine looks like:

	my_callback( _seconds, _data ):
		_seconds is how many seconds since last callback
		_data is whatever else was passed along to add()

	"""

	if not _name:
		global unique_id
		_name = "internal %d" % unique_id
		unique_id += 1

	if isinstance( _count, int ):
		if _count<2:
			repeat = None
		else:
			repeat = _count-1
	else:
		repeat = -1

	events[ _name ]  = {
		'name' : _name,
		'interval' : _seconds, 
		'next' : _seconds,
		'callback' : _callback,
		'repeat' : repeat,
		'data' : _data
	}


def remove( _name ):
	"""
	remove an existing event, if it's in there
	"""

	try:
		del events[ _name ]
	except KeyError:
		pass

def remove_all():
	"""
	remove ALL events.
	This is extreme, but useful for large mode transitions.
	"""
	global events
	events = {}
	

def pump( _seconds ):
	"""
	pump the events queue, do any new events
	_seconds is elapsed decimal time since last call
	"""

	# subtract the elapsed time from all events,
	#     add all the expired ones to a list
	expired = []
	for name, event in events.items():
		time = 	event['next'] - _seconds
		if time<=0.0:
			expired += [ ( time, event ) ]
		else:
			event['next'] = time


	# This isn't completely right - we will repeatedly call any repeats if
	# enough time has passed that they would repeatedly fire, but we place
	# them at the end of the list, so if the event sequence should be
	#    A B A C A D (A is repeated) we instead call A B C D A A
	# This could be fixed, but at expense of some time, and I don't need it

	while expired:
		# list is [] of (time,event), so we are sorting by expected time.
		# things that expired 'first' should be called first
		expired.sort()


		# now, for every event...
		new_expired = []
		for ( time, event ) in expired:

			# call the callback, tell it how many seconds it's been and give
			# it any extra data, if any
			event['callback']( event['interval']-time, event['data'] )

			# repeat?
			if event['repeat']:
				# next event is interval + time (which is negative), giving
				# remaining time
				time =  event['interval']+time
				event['next'] = time

				# decrement repeat count ( but not if -1 )
				if event['repeat']>0:
					event['repeat'] -= 1

				# if time is <= 0 then we trigger again now
				if time<=0.0:
					new_expired += [ ( time, event ) ]
			else:
				# This could fail IF one of the callbacks does a remove() I think.
				try:
					del events[ event['name'] ]
				except KeyError:
					pass

		# any repeats?
		expired = new_expired
